﻿#region Using

using System;
using System.Web;
using System.Web.Caching;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using BlogEngine.Core;

#endregion

namespace Controls
{
	/// <summary>
	/// Builds a month list.
	/// </summary>
	public class MonthDropDownList : Control
	{
		private const double CacheTimeoutInHours = 1;

        static MonthDropDownList()
		{
			Post.Saved += Post_Saved;
		}

		static void Post_Saved(object sender, SavedEventArgs e)
		{
			// invalidate cache whenever a post is modified
			lock (_SyncRoot)
			{
				HttpRuntime.Cache.Remove(CacheKey);
			}

		}

		#region Private fields

		private readonly static object _SyncRoot = new object();
        private const string CacheKey = "BE_MonthDropDownListCacheKey";
		#endregion

		private static SortedDictionary<DateTime, int> GetPostsPerMonth()
		{
			lock (_SyncRoot)
			{
				SortedDictionary<DateTime, int> months = HttpRuntime.Cache[CacheKey] as SortedDictionary<DateTime, int>;
				if (months == null)
				{
					months = new SortedDictionary<DateTime, int>();
					// let dictionary expire after 1 hour
					HttpRuntime.Cache.Insert(CacheKey, months, null, DateTime.Now.AddHours(CacheTimeoutInHours), Cache.NoSlidingExpiration);

					foreach (Post post in Post.Posts)
					{
						if (post.IsVisibleToPublic)
						{
							DateTime month = new DateTime(post.DateCreated.Year, post.DateCreated.Month, 1);
							int count;
							// if the date is not in the dictionary, count will be set to 0
							months.TryGetValue(month, out count);
							++count;
							months[month] = count;
						}
					}
				}
				return months;
			}
		}

		private string RenderMonths()
		{
			SortedDictionary<DateTime, int> months = GetPostsPerMonth();
			if (months.Keys.Count == 0)
				return "<p>" + Resources.labels.none + "</p>";

            HtmlGenericControl sel = new HtmlGenericControl("select");
            sel.Attributes.Add("id", "monthDropDownList");
            sel.Attributes.Add("onchange", "document.location.href=this.options[this.selectedIndex].value;");
            HtmlGenericControl monthYear = null;

            foreach (DateTime date in months.Keys)
            {
                monthYear = new HtmlGenericControl("option");
                monthYear.Attributes.Add("value", string.Format("{0}{1}/default.aspx", Utils.AbsoluteWebRoot, date.ToString("yyyy-MM").Replace("-", "/")));
                monthYear.InnerText = date.ToString("yyyy年MM月") + "(" + months[date].ToString() + ")";
                sel.Controls.AddAt(0, monthYear);
            }

            monthYear = new HtmlGenericControl("option");
            monthYear.Attributes.Add("value", string.Empty);
            monthYear.InnerText = "-按月归档-";
            sel.Controls.AddAt(0, monthYear);

			System.IO.StringWriter sw = new System.IO.StringWriter();
            sel.RenderControl(new HtmlTextWriter(sw));
			return sw.ToString();
		}

		private SortedDictionary<string, Guid> SortGategories(Dictionary<Guid, string> categories)
		{
			SortedDictionary<string, Guid> dic = new SortedDictionary<string, Guid>();
			foreach (Guid key in categories.Keys)
			{
				dic.Add(categories[key], key);
			}

			return dic;
		}

		/// <summary>
		/// Renders the control.
		/// </summary>
		public override void RenderControl(HtmlTextWriter writer)
		{
			if (Post.Posts.Count > 0)
			{
				string html = RenderMonths();
				writer.Write(html);
				writer.Write(Environment.NewLine);
			}
		}
	}
}